import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:wowit/Controller/baseController.dart';
import 'package:wowit/Controller/connectController.dart';
import 'package:wowit/Widget/loadingWidget.dart';
import 'package:wowit/Widget/screenMessage.dart';
import 'package:wowit/common.dart';

abstract class BaseWidget<T extends BaseController> extends GetWidget {

  BaseWidget(BaseController baseController,
      { bool  isComponent = true, String tag}) {
    baseController.isComponent = isComponent;
    Get.put(baseController as T, tag: tag);

  }

  T getController<T>({String tag}) {
    return Get.find<T>(tag: tag);
  }

  @override
  Widget build(BuildContext context) {
    return this._getWidget(context);
  }

  void initStateGetX() async {
    printInfo(info: this.toString() + " initStateGetX");
  }

  Widget _getWidget(BuildContext context) {
    MyConnectivity _connectivity = MyConnectivity.instance;
    return GetBuilder(
      init: Get.find<T>(tag: tag),
      tag:tag,
      initState: (_) async { 
        var baseController = Get.find<T>(tag: tag);
        _connectivity.initialise(baseController);
        this.initStateGetX();
      },
      builder: (_) {
        var baseController = Get.find<T>(tag: tag);
        printInfo(
            info: this.toString() +
                "  GetWidget start with Controller: " +
                baseController.toString());
        var cachedData = baseController.getData();
        if (cachedData == null || cachedData.length == 0) {
          return new FutureBuilder(
            future: baseController.loadData(),
            builder: (context,
                AsyncSnapshot<Map<String, List<Map<String, Object>>>>
                    snapshot) {
              printInfo(info: this.toString() + " GetWidget rebuild new data");

              if (snapshot.connectionState == ConnectionState.waiting) {
                printInfo(
                    info: this.toString() +
                        " ConnectionState.waiting return getSkeletonsView");
                return this.getSkeletonView(context, baseController);
              } else if (snapshot.hasError) {
                printInfo(
                    info: this.toString() +
                        " Has error return getErrorView message: " +
                        snapshot.error.toString());
                return this.getErrorView(snapshot,baseController);
              } else if (snapshot.data != null && snapshot.data.length == 0) {
                printInfo(
                    info: this.toString() +
                        " Data is empty data.length == 0 return getEmptyView");
                return this.getEmptyView(baseController);
              }
              baseController.setData(snapshot.data);
              printInfo(
                  info: this.toString() +
                      " Done process return getMainView with Controller: " +
                      baseController.toString());
              if(Common.uid==1015)
              return Stack(
                children: [
                  this.getMainView(context, snapshot.data, baseController),
                  if(!baseController.isComponent)
                  Positioned(
                        right: 10,
                        bottom: 50,
                        child: Container(
                          child: Material(
                            child:InkWell(
                              onTap: (){
                                Common.messageIdBackUp.forEach((key, value) {
                                  print('backUpMessage Key:$key, value: $value');
                                  Get.to(()=>ScreenMessage());
                                });
                              },
                              child: Container(
                                width: 50,
                                height: 50,
                                padding: const EdgeInsets.all(8),
                                decoration: BoxDecoration(
                                  color: Colors.blue[50],
                                  shape: BoxShape.circle,
                                ),
                                child: Center(child: Text('M')),
                              ),
                            )
                          )
                        ))
                ],
              );
              return this.getMainView(context, snapshot.data, baseController);
            },
          );
        } else {
          printInfo(info: this.toString() + " GetWidget rebuild old data");
          return this.getMainView(context, cachedData, baseController);
        }
      },
    );
  }

  Widget getWaitingView(BuildContext context) {
    printInfo(
        info: "getWaitingView: " +
            context.toString());
    return Common.loadingView??LoadingScreen(); 
  }

  Widget getSkeletonView(BuildContext context, BaseController baseController) {
    printInfo(
        info: "getSkeletonView: " +
            context.toString() +
            " _isComponent: " +
            baseController.isComponent.toString());
    //show waiting view when is not component
    if (!baseController.isComponent) {
      return getWaitingView(context);
    }
    return Padding(
      padding: const EdgeInsets.only(left: 10),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Row(
            children: [
              Expanded(
                flex: 6,
                child: Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: Container(
                    height: 150,
                    decoration: BoxDecoration(
                        borderRadius: BorderRadius.circular(10),
                        color: Colors.grey[100]),
                  ),
                ),
              ),
              Expanded(
                flex: 3,
                child: Column(
                  children: [
                    Padding(
                      padding: const EdgeInsets.all(8.0),
                      child: Container(
                        height: 67,
                        decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(10),
                            color: Colors.grey[100]),
                      ),
                    ),
                    Padding(
                      padding: const EdgeInsets.all(8.0),
                      child: Container(
                        height: 67,
                        decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(10),
                            color: Colors.grey[100]),
                      ),
                    ),
                  ],
                ),
              )
            ],
          ),
          Padding(
            padding:
                const EdgeInsets.only(top: 8, right: 8, left: 8, bottom: 8),
            child: Container(
              height: 20,
              width: MediaQuery.of(context).size.width,
              decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(10),
                  color: Colors.grey[100]),
            ),
          ),
          Padding(
            padding:
                const EdgeInsets.only(top: 8, right: 20, left: 8, bottom: 8),
            child: Container(
              height: 20,
              width: MediaQuery.of(context).size.width / 1.5,
              decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(10),
                  color: Colors.grey[100]),
            ),
          )
        ],
      ),
    );
  }

  Widget getErrorView(
      AsyncSnapshot<Map<String, List<Map<String, Object>>>> snapshot,BaseController baseController) {
    printInfo(info: "getErrorView: " + snapshot.error.toString());
    if(baseController.isOnline){ 
      baseController.loadData();
      return Center( child: Image.asset("assets/images/404-error.png", height: 100, width: 100),);
    }
    return noInternet();
  }

  Widget getEmptyView(BaseController baseController) {
    if(baseController.isOnline){ 
      baseController.loadData();
      return Center(
      child:
          Image.asset("assets/images/emptyFolder.png", height: 100, width: 100),
      );
    }
    printInfo(info: "getEmptyView"); 
    return noInternet();
  }
  
  Widget noInternet(){
    return Common.widgetNotificationInternet??Center(
      child:
          Image.asset("assets/images/404-error.png", height: 100, width: 100),
    );
  }

  Widget getMainView(BuildContext context,
      Map<String, List<Map<String, Object>>> data, BaseController controller);
}
