part of edit_screen;

class EditorController {
  final List<EditorItem> editorItems;
  final Map<String, Object> dataEntity = {};
  final String onSaveProcedure;
  String _messageInvalidDialog;
  bool _isDirtyData = false;
  RxBool _isConfirm = false.obs;
  RxBool _isWaiting = false.obs;
  RxBool _succeed = RxBool(false);
  RxBool _isShowInValidDialog = false.obs;
  VoidCallback onSuccess;
  static Map<String, Object> mapDataReturn;

  EditorController(
    Map<String, Object> dataEntity,
    this.onSaveProcedure,
    this.editorItems,
  ) {
    _addDefaultValues();
    initalValue(dataEntity);
  }

  void initalValue(Map<String, Object> initialMapValue) {
    if (initialMapValue != null) {
      dataEntity.addEntries(initialMapValue.entries);
    }
  }

  void onValueChanged(String fieldName, Object value) {
    Object _value = value;
    final editorItem = _getEditorItemByFieldName(fieldName);
    if (editorItem?.type == EEditorType.int && (value is String)) {
      _value = int.tryParse(value);
    } else if (editorItem?.type == EEditorType.double && (value is String)) {
      _value = double.tryParse(value);
    } else if (editorItem?.type == EEditorType.money && (value is String)) {
      _value = double.tryParse(value.replaceAll(RegExp(','), ''));
    }

    dataEntity.update(
      fieldName,
      (_) => _value,
      ifAbsent: () => _value,
    );
    _isDirtyData = true;
  }

  void confirmAndSave({VoidCallback onSuccess}) {
    if (onSuccess != null) {
      this.onSuccess = onSuccess;
    }
    _isConfirm.value = true;
  }

  void showInvalidDialog({String message}) {
    _messageInvalidDialog = message;
    _isShowInValidDialog.value = true;
  }

  Future<Map<String, Object>> save({
    VoidCallback onSuccess,
    bool useLoadingDialog = false,
    Function(bool success, Map<String, List<Map<String, Object>>> response)
        afterSave,
  }) async {
    Map<String, Object> dataReturn = {};
    if (onSuccess != null) {
      this.onSuccess = onSuccess;
    }
    if (_isDirtyData) {
      final Map<String, Object> jParam = dataEntity.map((key, value) {
        if (value is DateTime) {
          value = value.toString();
        }
        return MapEntry('@$key', value);
      });

      jParam.removeWhere(
          (key, value) => _checkIsImagesKey(key.substring(1, key.length)));

      try {
        if (useLoadingDialog) _isWaiting.value = true;
        print('EditorWidget: Submitting...');

        final response =
            await SQLService.execute(onSaveProcedure, params: jParam);
        print('result: $response');
        dataReturn = response;
        if (response['Exception']?.isNotEmpty ?? false) {
          _succeed.value = false;
          afterSave?.call(false, response);
          print(
              'Editor Save Error On Server: ${response['Exception']?.first['Message']}');
        } else {
          await _saveImages(response);
          _succeed.value = true;
          afterSave?.call(true, response);
          onSuccess?.call();
        }
      } catch (e, stackTrace) {
        print('Editor Save Error: $e');
        print(stackTrace);
      } finally {
        print('finally Save Error:');
      }
      mapDataReturn = dataReturn;
      return dataReturn;
    } else {
      print('data have already save');
      mapDataReturn = dataReturn;
      return dataReturn;
    }
  }

// #region TextField
  String getHintText(String fieldName) {
    final editorItem = _getEditorItemByFieldName(fieldName);
    if (editorItem?.hint != null && editorItem.hint.isNotEmpty) {
      return editorItem.hint;
    }
    if (editorItem?.type == EEditorType.money) {
      return Message.getMessage('EditorWidget_nhap_so_tien');
    }
    if (editorItem?.title != null && editorItem.title.isNotEmpty) {
      return Message.getMessage('EditorWidget_nhap') +
          ' ' +
          _removeLastStarSymbolAndLowCase(editorItem.title);
    }
    return Message.getMessage('EditorWidget_nhap_thong_tin');
  }

  String getText(String fieldName) {
    final editorItem = _getEditorItemByFieldName(fieldName);

    if (editorItem == null || dataEntity[fieldName] == null) return '';

    if (editorItem.type == EEditorType.money) {
      return PriceInputFormatter.format(
          (dataEntity[fieldName] as double).toInt().toString());
    }

    return dataEntity[fieldName].toString();
  }

  String getValidator(String fieldName) {
    final editorItem = _getEditorItemByFieldName(fieldName);

    bool isRequired = editorItem?.isRequired ?? false;

    if (!isRequired) return null;

    if (editorItem.validator != null && dataEntity[fieldName] != null) {
      return editorItem.validator(dataEntity[fieldName]);
    }

    switch (editorItem.type) {
      case EEditorType.image:
        return _validateImage(dataEntity[fieldName])
            ? null
            : _getEmptyMessageValidator(editorItem);
      case EEditorType.string:
        return _validateString(dataEntity[fieldName])
            ? null
            : _getEmptyMessageValidator(editorItem);
      case EEditorType.phone:
        return Methods.validPhoneNumber(dataEntity[fieldName] ?? '')
            ? null
            : _getInvalidMessageValidator(
                Message.getMessage('EditorWidget_so_dien_thoai'));
      case EEditorType.email:
        return (dataEntity[fieldName] as String).isEmail
            ? null
            : _getInvalidMessageValidator(
                Message.getMessage('EditorWidget_email'));
      case EEditorType.int:
        return dataEntity[fieldName] != null
            ? null
            : _getEmptyMessageValidator(editorItem);
      case EEditorType.double:
        return dataEntity[fieldName] != null
            ? null
            : _getEmptyMessageValidator(editorItem);
      case EEditorType.money:
        return _validateMoney(dataEntity[fieldName])
            ? null
            : _getEmptyMessageValidator(editorItem);
        break;
      case EEditorType.datetime:
        return _validateDateTime(dataEntity[fieldName])
            ? null
            : _getEmptyMessageValidator(editorItem);
      default:
        return dataEntity[fieldName] != null
            ? null
            : _getEmptyMessageValidator(editorItem);
    }
  }

  TextInputType getTextInputType(String fieldName) {
    final editorItem = _getEditorItemByFieldName(fieldName);

    if (editorItem == null) return null;

    if (editorItem.type == EEditorType.int ||
        editorItem.type == EEditorType.double ||
        editorItem.type == EEditorType.money ||
        editorItem.type == EEditorType.phone) {
      return TextInputType.number;
    }
    if (editorItem.type == EEditorType.email) {
      return TextInputType.emailAddress;
    }
    return null;
  }

  List<TextInputFormatter> getInputFormatters(String fieldName) {
    final editorItem = _getEditorItemByFieldName(fieldName);

    if (editorItem == null) return <TextInputFormatter>[];

    if (editorItem.type == EEditorType.money) {
      return <TextInputFormatter>[
        FilteringTextInputFormatter.digitsOnly,
        PriceInputFormatter(),
      ];
    }

    if (editorItem.type == EEditorType.int ||
        editorItem.type == EEditorType.phone) {
      return <TextInputFormatter>[
        FilteringTextInputFormatter.digitsOnly,
      ];
    }

    return <TextInputFormatter>[];
  }

  String _getEmptyMessageValidator(EditorItem item) {
    if (item.invalidMessage != null && item.invalidMessage != "") {
      return item.invalidMessage;
    }
    // if (item.title != null && item.title != "") {
    //   return null;
    // }
    return Message.getMessage('EditorWidget_vui_long_nhap') +
        ' ' +
        (item.title != null
            ? _removeLastStarSymbolAndLowCase(item.title)
            : Message.getMessage('EditorWidget_thong_tin'));
  }

  String _getInvalidMessageValidator(String title) {
    return Message.getMessage('EditorWidget_vui_long_nhap') +
        ' ' +
        (title != null
            ? _removeLastStarSymbolAndLowCase(title)
            : Message.getMessage('EditorWidget_thong_tin')) +
        ' ' +
        Message.getMessage('EditorWidget_hop_le');
  }

// #endregion TextField

// #region Title
  String _removeLastStarSymbolAndLowCase(String str) {
    if (str != null) {
      return str.replaceAll('*', '').toLowerCase();
    }
    return '';
  }

  String getTitleByFieldName(String fieldName) {
    return _getEditorItemByFieldName(fieldName)?.title;
  }

// #endregion Title

// #region Image
  bool _checkIsImagesKey(String fieldName) {
    final editorItem = _getEditorItemByFieldName(fieldName);
    if (editorItem != null && editorItem.type == EEditorType.image) {
      return true;
    }
    return false;
  }

  Future<void> _saveImages(
      Map<String, List<Map<String, Object>>> response) async {
    final imageEditorItems = editorItems
        .where((element) => element.type == EEditorType.image)
        .toList();

    if (imageEditorItems.isEmpty) return;

    final futures = <Future>[];

    for (final imageEditorItem in imageEditorItems) {
      var value = dataEntity[imageEditorItem.fieldName];
      if (value == null) continue;

      List<File> images = [];
      if (value is List<File>) {
        images.addAll(value);
      } else if (value is File) {
        images.add(value);
      } else if (value is Map<String, Object>) {
        File file = value.values.firstWhere(
          (e) => e is File,
          orElse: () => null,
        );
        if (file != null) {
          images.add(file);
        }
      } else if (value is List<Map<String, Object>>) {
        value.forEach((e) {
          File file = e.values.firstWhere(
            (e) => e is File,
            orElse: () => null,
          );
          if (file != null) {
            images.add(file);
          }
        });
      }
      UploadImageInfo uploadImageInfo = imageEditorItem.uploadImageInfo;
      for (var data in response[uploadImageInfo.responseIdsName]) {
        if (data == null) continue;
        int id = Methods.getInt(data, 'ID');
        if (id > 0) {
          for (File image in images) {
            Future<String> upload;
            if (uploadImageInfo.parentTableName != null &&
                uploadImageInfo.parentTableName.isNotEmpty &&
                uploadImageInfo.parentTableName != uploadImageInfo.tableName) {
              upload = Methods.uploadFileParentChild(
                image,
                uploadImageInfo.tableName,
                -1,
                uploadImageInfo.fieldName,
                uploadImageInfo.parentTableName,
                id,
              );
            } else {
              upload = Methods.uploadFileParent(
                image,
                uploadImageInfo.tableName,
                id,
                uploadImageInfo.fieldName,
              );
            }
            futures.add(
              upload.then((value) {
                print("Result upload image: $value");
                if (response[uploadImageInfo.tableName] == null) {
                  response[uploadImageInfo.tableName] = <Map<String, Object>>[];
                }
                var list = response[uploadImageInfo.tableName];
                list.add({
                  'ID': Methods.getFileId(value),
                  'refID': id,
                  'hinhAnh_image_path': Methods.getFilePath(value),
                });
              }, onError: (err) {
                print('Error upload image: $err');
              }),
            );
          }
        }
      }
    }

    await Future.wait(futures);
  }

// #endregion Image

// #region Validate Data
  bool validate({int step, bool isShowInvalidDialog = true}) {
    if (!_checkValidData(step)) {
      print('validate');
      if (isShowInvalidDialog == true) {
        _isShowInValidDialog.value = true;
      }
      return false;
    } else {
      return true;
    }
  }

  bool _checkValidData(int step) {
    for (final editorItem in editorItems) {
      if ((step != null && editorItem.step != step) || !editorItem.isRequired) {
        continue;
      }
      if (getValidator(editorItem.fieldName) != null) {
        _messageInvalidDialog = _getEmptyMessageValidator(editorItem);
        return false;
      }
    }
    return true;
  }

  bool _validateString(dynamic value) {
    return (value is String) && value.isNotEmpty;
  }

  bool _validateMoney(dynamic value) {
    return (value is num && value > 0);
  }

  bool _validateDateTime(dynamic value) {
    return value is DateTime && value != null;
  }

  // #endregion Validate Data

  void _resetAfterDone() {
    _isWaiting.value = false;
    _succeed.value = null;
    _isDirtyData = false;
    _addDefaultValues();
  }

  void _resetAfterFail() {
    _isWaiting.value = false;
    _succeed.value = false;
    _isDirtyData = true;
  }

  void _addDefaultValues() {
    editorItems.forEach((item) {
      dataEntity.addAll({item.fieldName: item.defaultValue});
    });
  }

  EditorItem _getEditorItemByFieldName(String fieldName) {
    final editorItem = editorItems.firstWhere(
        (element) => element.fieldName == fieldName,
        orElse: () => null);
    return editorItem;
  }

  bool _validateImage(List images) {
    if (images != null && images.isNotEmpty) {
      return true;
    }
    return false;
  }
}
