part of edit_screen;

class LoadingEditorDialog extends StatefulWidget {
  final EditorController editorController;

  const LoadingEditorDialog({Key key, this.editorController}) : super(key: key);

  @override
  _LoadingEditorDialogState createState() => _LoadingEditorDialogState();
}

class _LoadingEditorDialogState extends State<LoadingEditorDialog>
    with TickerProviderStateMixin {
  AnimationController _loadingAnimationController;
  AnimationController _doneController;
  AnimationController _iconAnimationController;

  Tween<double> loadingTween, doneTween;
  ColorTween failedColorTween, succeedColorTween;

  Animation<double> loadinngAnimation, completedAnimation, iconAnimation;
  Animation<Color> failedColorAnimation, succeedColorAnimation;

  final loadingColor = Color(0xff2addd5);
  bool isDirty = true;

  @override
  void initState() {
    print('initState LoadingEditorDialog');
    _loadingAnimationController = AnimationController(
      vsync: this,
      duration: Duration(milliseconds: 500),
    );

    _doneController = AnimationController(
      vsync: this,
      duration: Duration(milliseconds: 400),
    );

    _iconAnimationController = AnimationController(
      vsync: this,
      duration: Duration(milliseconds: 500),
    );

    loadingTween = Tween<double>(begin: 0.0, end: 1.0);
    doneTween = Tween<double>(begin: 0.2, end: 1.0);
    failedColorTween = ColorTween(begin: loadingColor, end: Color(0xFFF44336));
    succeedColorTween = ColorTween(begin: loadingColor, end: Color(0xFF4CAF50));

    loadinngAnimation = loadingTween.animate(_loadingAnimationController);
    completedAnimation = doneTween.animate(_doneController);
    failedColorAnimation = failedColorTween.animate(_iconAnimationController);
    succeedColorAnimation = succeedColorTween.animate(_iconAnimationController);
    iconAnimation = loadingTween.animate(
      CurvedAnimation(
        parent: _iconAnimationController,
        curve: Curves.bounceOut,
      ),
    );
    widget.editorController._succeed.listen((isSucceed) {
      if (isSucceed != null && isDirty) {
        runDoneAnimation(isSucceed);
      }
    });

    _loadingAnimationController.addListener(() => setState(() {}));
    _loadingAnimationController.repeat();

    super.initState();
  }

  void runDoneAnimation(bool isSucceed) async {
    if (isDirty) {
      print('runDoneAnimation');
      _doneController?.forward();

      await Future.delayed(Duration(milliseconds: 300));
      _iconAnimationController.forward();

      await Future.delayed(Duration(milliseconds: 500));
      _loadingAnimationController.stop();

      _loadingAnimationController.dispose();
      _doneController.dispose();

      _iconAnimationController.dispose();

      if (isSucceed) {
        widget.editorController.onSuccess?.call();
      }
    }
    isDirty = false;
  }

  @override
  Widget build(BuildContext context) {
    return Center(
      child: Container(
        height: 160.0 +
            ((widget.editorController._succeed.value == false)
                ? iconAnimation.value * 40.0
                : iconAnimation.value * 10.0),
        width: 180,
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.all(
            Radius.circular(16),
          ),
        ),
        child: Scaffold(
          backgroundColor: Colors.transparent,
          body: Center(
            child: Padding(
              padding: const EdgeInsets.only(
                top: 16,
                bottom: 0,
                left: 16,
                right: 16,
              ),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisSize: MainAxisSize.min,
                children: [
                  ConstrainedBox(
                    constraints: BoxConstraints.tight(
                      Size(68, 68),
                    ),
                    child: CustomPaint(
                      painter: Circle(
                          loadinngAnimation.value,
                          completedAnimation.value,
                          widget.editorController._succeed.value ?? true
                              ? succeedColorAnimation.value ?? Colors.white
                              : failedColorAnimation.value ?? Colors.white),
                      child: widget.editorController._succeed.value != null
                          ? Icon(
                              widget.editorController._succeed.value ?? false
                                  ? Icons.check_rounded
                                  : Icons.close,
                              size: iconAnimation.value * 28,
                              color: widget.editorController._succeed.value ??
                                      false
                                  ? Colors.green
                                  : Colors.red,
                            )
                          : SizedBox(),
                    ),
                  ),
                  SizedBox(
                      height: widget.editorController._succeed.value != null
                          ? 12
                          : 18),
                  Opacity(
                    opacity: widget.editorController._succeed.value != null
                        ? (iconAnimation.value)
                        : 1 - iconAnimation.value,
                    child: Text(
                      widget.editorController._succeed.value == null
                          ? Message.getMessage('EditorWidget_dang_luu') + '...'
                          : widget.editorController._succeed.value
                              ? Message.getMessage(
                                  'EditorWidget_luu_thanh_cong')
                              : Message.getMessage('EditorWidget_luu_that_bai'),
                      style: TextStyle(
                        fontSize: 14,
                        color: Colors.black.withOpacity(0.7),
                        fontWeight: FontWeight.w600,
                      ),
                      overflow: TextOverflow.ellipsis,
                    ),
                  ),
                  if (widget.editorController._succeed.value == false)
                    SizedBox(height: 12),
                  if (widget.editorController._succeed.value == false)
                    Opacity(
                      opacity: widget.editorController._succeed.value != null
                          ? (iconAnimation.value)
                          : 1 - iconAnimation.value,
                      child: Text(
                        Message.getMessage(
                            'EditorWidget_vui_long_kiem_tra_internet'),
                        style: TextStyle(
                          fontSize: 12,
                          color: Colors.black45,
                          fontWeight: FontWeight.w600,
                        ),
                      ),
                    ),
                  Spacer(),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      SizedBox(
                        height: iconAnimation.value * 38,
                        child: TextButton(
                          style: TextButton.styleFrom(),
                          onPressed: () {
                            if (widget.editorController._succeed.value) {
                              widget.editorController.onSuccess?.call();
                            }
                            Get.back();
                          },
                          child: Text(
                            'OK',
                            style: TextStyle(
                              fontSize: 13,
                              color: Colors.greenAccent.withOpacity(0.8),
                              fontWeight: FontWeight.bold,
                            ),
                          ),
                        ),
                      ),
                      if (widget.editorController._succeed.value == false)
                        Spacer(),
                      if (widget.editorController._succeed.value == false)
                        SizedBox(
                          height: iconAnimation.value * 38,
                          child: TextButton(
                            style: TextButton.styleFrom(),
                            onPressed: () {
                              Get.back();
                              // Get.offAll(NavigationBar());
                            },
                            child: Text(
                              Message.getMessage('EditorWidget_thu_lai'),
                              style: TextStyle(
                                fontSize: 13,
                                color: Colors.redAccent.withOpacity(0.8),
                                fontWeight: FontWeight.bold,
                              ),
                            ),
                          ),
                        ),
                    ],
                  ),
                  SizedBox(height: 8),
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }
}

class Circle extends CustomPainter {
  final double time;
  final double percent;
  final Color color;
  final pi = 3.14;

  Circle(this.time, this.percent, this.color);

  @override
  void paint(Canvas canvas, Size size) {
    Offset center = Offset(size.width / 2, size.height / 2);
    double radius = 24;
    Paint circle = Paint()
      ..strokeWidth = 6
      ..color = Color(0xffb2c2e0).withOpacity(0.8)
      ..style = PaintingStyle.stroke;
    canvas.drawCircle(center, radius, circle);

    Paint arc = Paint()
      ..strokeWidth = 6
      ..color = color
      ..style = PaintingStyle.stroke
      ..strokeCap = StrokeCap.square;

    double angle = 2 * pi * percent;

    canvas.drawArc(
      Rect.fromCircle(center: center, radius: radius),
      pi * 2 * time,
      angle,
      false,
      arc,
    );
    // print('time: $time, percent: $percent, color: $color');
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}
