import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:wowit/Controller/filter_controller.dart';
import 'package:wowit/message.dart';

class FilterScreen extends StatefulWidget {
  FilterScreen({
    Key key,
    @required this.title,
    @required this.getConditionView,
    @required this.onApply,
    @required this.onClear,
    this.color,
    this.clearTextStyle,
    this.applyTextStyle,
    this.clearBackgroundColor,
    this.applyBackgroundColor,
    this.clearText,
    this.applyText,
    this.filterController,
  }) : super(key: key);

  final String title;
  final Widget Function(
    Map<String, Object> mapCondition,
    void Function(String fieldName, Object value) onValueChange,
  ) getConditionView;
  final FilterController filterController;
  final VoidCallback onApply;
  final VoidCallback onClear;
  //Custom
  final Color color;
  final String clearText;
  final String applyText;
  final TextStyle clearTextStyle;
  final TextStyle applyTextStyle;
  final Color clearBackgroundColor;
  final Color applyBackgroundColor;

  @override
  _FilterScreenState createState() => _FilterScreenState();
}

class _FilterScreenState extends State<FilterScreen> {
  Widget conditionWidget;
  @override
  void initState() {
    conditionWidget = widget.getConditionView(
      widget.filterController.mapCondition,
      widget.filterController.updateMapCondition,
    );
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        automaticallyImplyLeading: false,
        centerTitle: true,
        backgroundColor: Colors.white,
        elevation: 0.0,
        leading: IconButton(
          onPressed: () => Get.back(),
          icon: Icon(
            Icons.arrow_back_ios_outlined,
            color: widget.color ?? Color(0xff2a367e),
          ),
        ),
        title: Text(
          widget.title ?? '',
          style: TextStyle(
              color: widget.color ?? Color(0xff2a367e),
              fontWeight: FontWeight.w600),
        ),
      ),
      body: Container(
        color: Colors.white,
        child: Column(
          children: [
            Expanded(
              child: conditionWidget,
            ),
            /* Container(
              height: 60.0,
              width: MediaQuery.of(context).size.width,
              decoration: BoxDecoration(
                color: Colors.white,
                boxShadow: [
                  BoxShadow(
                    color: Color(0xff2a367e),
                    spreadRadius: 5,
                    blurRadius: 5,
                    offset: Offset(0, 5),
                  ),
                ],
              ),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  Expanded(
                    child: FilterButton(
                      onTap: () {
                        setState(() {
                          widget.onClear();
                          conditionWidget = widget.getConditionView(
                            widget.filterController.mapCondition,
                            widget.filterController.updateMapCondition,
                          );
                        });
                      },
                      text: Message.getMessage("LocBDS_Button_ChonLai"),
                      textStyle:
                          TextStyle(color: widget.color ?? Color(0xff2a367e)),
                      backgroundColor: Colors.white,
                    ),
                  ),
                  Container(
                    width: 0.2,
                    color: Color(0xff2a367e),
                  ),
                  Expanded(
                    child: FilterButton(
                      onTap: widget.onApply,
                      backgroundColor: widget.color,
                      text: Message.getMessage("LocBDS_Button_ApDung"),
                    ),
                  ),
                ],
              ),
            ), */
            Padding(
              padding: const EdgeInsets.only(left: 20, right: 20),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  Expanded(
                    child: InkWell(
                      onTap: () {
                        setState(() {
                          widget.onClear();
                          conditionWidget = widget.getConditionView(
                            widget.filterController.mapCondition,
                            widget.filterController.updateMapCondition,
                          );
                        });
                      },
                      child: Container(
                          height: 45,
                          decoration: BoxDecoration(
                              color: Colors.white,
                              border: new Border.all(
                                color: Color(0xff2a367e),
                                width: 1.5,
                              ),
                              borderRadius: BorderRadius.circular(10)),
                          child: Center(
                            child: Text(
                              Message.getMessage('SearchMap_Datlai'),
                              style: TextStyle(
                                  color: Color(0xff2a367e),
                                  fontWeight: FontWeight.w600),
                            ),
                          )),
                    ),
                  ),
                  SizedBox(
                    width: 10,
                  ),
                  Expanded(
                    child: InkWell(
                      onTap: () {
                        widget.onApply();
                      },
                      child: Container(
                          height: 45,
                          decoration: BoxDecoration(
                              color: Color(0xff2a367e),
                              borderRadius: BorderRadius.circular(10)),
                          child: Center(
                              child: Text(
                            Message.getMessage('LocBDS_Button_ApDung'),
                            style: TextStyle(
                                color: Colors.white,
                                fontWeight: FontWeight.w600),
                          ))),
                    ),
                  )
                ],
              ),
            ),
            SizedBox(
              height: 20,
            )
          ],
        ),
      ),
    );
  }
}

class FilterButton extends StatelessWidget {
  const FilterButton({
    Key key,
    @required this.text,
    this.backgroundColor,
    this.onTap,
    this.textStyle,
  }) : super(key: key);

  final VoidCallback onTap;
  final Color backgroundColor;
  final String text;
  final TextStyle textStyle;

  @override
  Widget build(BuildContext context) {
    return InkWell(
      onTap: onTap,
      child: Container(
        color: backgroundColor ?? Color(0xff2a367e),
        alignment: Alignment.center,
        width: MediaQuery.of(context).size.width,
        height: MediaQuery.of(context).size.height,
        child: Text(
          text,
          style: textStyle ?? TextStyle(color: Colors.white),
        ),
      ),
    );
  }
}
