library seached_list;

import 'dart:async';

import 'package:flutter/material.dart';
import 'package:wowit/Controller/baseController.dart';
import 'package:wowit/Controller/load_more_base_controller.dart';
import 'package:wowit/Controller/searched_controller.dart';
import 'package:wowit/Widget/filter_widget/filter_widget.dart';
import 'package:wowit/Widget/filter_widget/util.dart';

part 'package:wowit/Controller/search_controller.dart';

// ignore: must_be_immutable
class SearchedList extends FilterWidget<_SearchController> {
  final SearchedListModel searchedListModel;
  final SearchedDataLoader searchedDataLoader;
  final String fieldNameSearching;

  SearchedList({
    @required this.searchedListModel,
    @required this.searchedDataLoader,
    @required this.fieldNameSearching,
  }) : super(
          _SearchController(searchedDataLoader),
          isFilterOnServer: searchedListModel.isFilterOnServer,
          title: searchedListModel.title,
          filterItems: searchedListModel.filterItems,
        );

  ScrollController scrollController;
  _SearchController _searchController;

  @override
  Widget getConditionView(Map<String, Object> mapCondition,
      void onValueConditionChanged(String fieldName, Object value)) {
    return searchedListModel.getConditionView(
        mapCondition, onValueConditionChanged);
  }

  void loadMore() {
    _searchController?.loadMore();
  }

  @override
  Widget getMainView(BuildContext context,
      Map<String, List<Map<String, Object>>> data, BaseController controller) {
    _searchController = controller as _SearchController;

    return _searchController.dataList.isNotEmpty
        ? Padding(
            padding: searchedListPadding ?? const EdgeInsets.all(0.0),
            child: ListView.builder(
              controller: scrollController,
              keyboardDismissBehavior: ScrollViewKeyboardDismissBehavior.onDrag,
              itemCount: _searchController.dataList.length + 1,
              itemBuilder: (BuildContext context, index) {
                if (index < _searchController.dataList.length) {
                  return searchedListModel
                      .getLineView(_searchController.dataList[index]);
                }
                if (_searchController.isLoadMoreRunning == true)
                  return getLoadingView();
                if (_searchController.hasNextPage == false && index > 20)
                  return getEndOfPageView();
                return SizedBox();
              },
            ),
          )
        : this.getEmptyView(_searchController);
  }

  Timer timer;
  void search(String keyword) { 
    if(timer!=null){
      timer.cancel();
    }
    timer = Timer(Duration(milliseconds: 500), () async { 
      onConditionValueChanged(fieldNameSearching, keyword);
    searchedListModel.isFilterOnServer
        ? applyFilterOnServer()
        : applyFilterBaseOnList();
      timer.cancel();
    });
  }
}

class SearchedListModel {
  final String title;
  final List<FilterItem> filterItems;
  final GetLineView getLineView;
  final GetConditionView getConditionView;
  final EdgeInsets searchedListPadding;
  final bool isFilterOnServer;

  SearchedListModel({
    this.title,
    this.filterItems,
    this.getLineView,
    this.getConditionView,
    this.searchedListPadding,
    this.isFilterOnServer,
  });
}
